@@ -94,6 +94,9 @@
 /* Define if you want RPA support compiled in */
 #undef RPA_ENABLE
 
+/* Define if you want NTLM authentication */
+#undef NTLM_ENABLE
+
 /* Define if you want SDPS support compiled in */
 #undef SDPS_ENABLE
 
@@ -109,6 +112,12 @@
 /* Define if you want GSSAPI authentication */
 #undef GSSAPI
 
+/* Define if you have HEIMDAL kerberos 5 */
+#undef HEIMDAL
+
+/* Define if you have MIT kerberos */
+#undef HAVE_GSS_C_NT_HOSTBASED_SERVICE
+
 /* Define if you want built-in SOCKS support */
 #undef HAVE_SOCKS
 
