@@ -74,7 +74,7 @@ static void numdump(const char *name, const int num)
 /* dump a numeric quantity at current indent */
 {
     indent('\0');
-    fprintf(stdout, "'%s':%d,\n", name, num);
+    fprintf(stdout, "'%s':%d,\n", name, NUM_VALUE_OUT(num));
 }
 
 static void booldump(const char *name, const int onoff)
@@ -132,8 +132,10 @@ void dump_config(struct runctl *runp, struct query *querylist)
      * We need this in order to know whether `interface' and `monitor'
      * are valid options or not.
      */
-#ifdef linux
+#if defined(linux)
     fputs("os_type = 'linux'\n", stdout);
+#elif defined(__FreeBSD__)
+    fputs("os_type = 'freebsd'\n", stdout);
 #else
     fputs("os_type = 'generic'\n", stdout);
 #endif
@@ -183,11 +185,13 @@ void dump_config(struct runctl *runp, struct query *querylist)
     indent('{');
 
     numdump("poll_interval", runp->poll_interval);
-    booldump("syslog", runp->use_syslog);
     stringdump("logfile", runp->logfile);
     stringdump("idfile", runp->idfile);
     stringdump("postmaster", runp->postmaster);
+    booldump("bouncemail", runp->bouncemail);
+    stringdump("properties", runp->properties);
     booldump("invisible", runp->invisible);
+    booldump("syslog", runp->use_syslog);
 
     if (!querylist)
     {
@@ -260,11 +264,11 @@ void dump_config(struct runctl *runp, struct query *querylist)
 	    stringdump("qvirtual", ctl->server.qvirtual);
  
 	    if (ctl->server.preauthenticate == A_KERBEROS_V4)
-		stringdump("auth", "kerberos_v4");
+		stringdump("preauth", "kerberos_v4");
 	    else if (ctl->server.preauthenticate == A_KERBEROS_V5)
-		stringdump("auth", "kerberos_v5");
+		stringdump("preauth", "kerberos_v5");
 	    else
-		stringdump("auth", "password");
+		stringdump("preauth", "password");
 
 #if defined(HAVE_GETHOSTBYNAME) && defined(HAVE_RES_SEARCH)
 	    booldump("dns", ctl->server.dns);
@@ -274,10 +278,10 @@ void dump_config(struct runctl *runp, struct query *querylist)
 	    listdump("aka", ctl->server.akalist);
 	    listdump("localdomains", ctl->server.localdomains);
 
-#ifdef linux
+#if defined(linux) || defined(__FreeBSD__)
 	    stringdump("interface", ctl->server.interface);
 	    stringdump("monitor", ctl->server.monitor);
-#endif /* linux */
+#endif /* linux || __FreeBSD__ */
 
 	    stringdump("plugin", ctl->server.plugin);
 	    stringdump("plugout", ctl->server.plugout);
@@ -325,9 +329,9 @@ void dump_config(struct runctl *runp, struct query *querylist)
 	stringdump("bsmtp", ctl->bsmtp);
 	indent('\0');
 	if (ctl->listener == LMTP_MODE)
-	    fputs("'lmtp':TRUE,", stdout);
+	    fputs("'lmtp':TRUE,\n", stdout);
 	else
-	    fputs("'lmtp':FALSE,", stdout);
+	    fputs("'lmtp':FALSE,\n", stdout);
 	    
 #ifdef INET6
 	stringdump("netsec", ctl->server.netsec);
