@@ -56,6 +56,7 @@ RETSIGTYPE
 sigchld_handler (int sig)
 /* process SIGCHLD to obtain the exit code of the terminating process */
 {
+    extern volatile int lastsig;		/* last signal received */
     pid_t pid;
 
 #if 	defined(HAVE_WAITPID)				/* the POSIX way */
@@ -77,6 +78,7 @@ sigchld_handler (int sig)
 
     wait(&status);
 #endif
+    lastsig = SIGCHLD;
 }
 
 int
@@ -109,7 +111,7 @@ daemonize (const char *logfile, void (*termhook)(int))
      group leader */
 
   if ((childpid = fork()) < 0) {
-    report(stderr, errno, "fork");
+    report(stderr, "fork (%s)\n", strerror(errno));
     return(PS_IOERR);
   }
   else if (childpid > 0) 
@@ -122,7 +124,7 @@ daemonize (const char *logfile, void (*termhook)(int))
 #if	defined(HAVE_SETSID)		/* POSIX */
   /* POSIX makes this soooo easy to do */
   if (setsid() < 0) {
-    report(stderr, errno, "setsid");
+    report(stderr, "setsid (%s)\n", strerror(errno));
     return(PS_IOERR);
   }
 #elif	defined(SIGTSTP)		/* BSD */
@@ -144,7 +146,7 @@ daemonize (const char *logfile, void (*termhook)(int))
   /* lose controlling tty */
   signal(SIGHUP, SIG_IGN);
   if ((childpid = fork()) < 0) {
-    report(stderr, errno, "fork");
+    report(stderr, "fork (%)\n", strerror(errno));
     return(PS_IOERR);
   }
   else if (childpid > 0) {
@@ -168,7 +170,7 @@ nottyDetach:
 
   /* Reopen stdin descriptor on /dev/null */
   if ((fd = open("/dev/null", O_RDWR)) < 0) {   /* stdin */
-    report(stderr, errno, "open: /dev/null");
+    report(stderr, "open: /dev/null (%s)\n", strerror(errno));
     return(PS_IOERR);
   }
 
@@ -176,11 +178,11 @@ nottyDetach:
     fd = open(logfile, O_CREAT|O_WRONLY|O_APPEND, 0666);	/* stdout */
   else
     if (dup(fd) < 0) {				/* stdout */
-      report(stderr, errno, "dup");
+      report(stderr, "dup (%s)\n", strerror(errno));
       return(PS_IOERR);
     }
   if (dup(fd) < 0) {				/* stderr */
-    report(stderr, errno, "dup");
+    report(stderr, "dup (%s)\n", strerror(errno));
     return(PS_IOERR);
   }
 
