@@ -16,7 +16,6 @@
 #endif
 #include <pwd.h>
 #include <string.h>
-#include <ctype.h>
 #ifdef HAVE_GETHOSTBYNAME
 #include <netdb.h>
 #endif /* HAVE_GETHOSTBYNAME */
@@ -33,22 +32,35 @@ extern char *program_name;
 void envquery(int argc, char **argv)
 /* set up basic stuff from the environment (including the rc file name) */
 {
-    struct passwd *pw;
+    struct passwd by_name, by_uid, *pwp;
 
-    if ((program_name = strrchr(argv[0], '/')) != NULL)
-	++program_name;
-    else
-	program_name = argv[0];
+    if (!(user = getenv("LOGNAME")))
+	user = getenv("USER");
 
-    if ((user = getenv("USER")) == (char *)NULL)
-        user = getenv("LOGNAME");
-
-    if ((user == (char *)NULL) || (home = getenv("HOME")) == (char *)NULL)
+    if (!(pwp = getpwuid(getuid())))
     {
-	if ((pw = getpwuid(getuid())) != NULL)
+	fprintf(stderr,
+		_("%s: You don't exist.  Go away.\n"),
+		program_name);
+	exit(PS_UNDEFINED);
+    }
+    else
+    {
+	memcpy(&by_uid, pwp, sizeof(struct passwd));
+	if (!user)
+	    pwp = &by_uid;
+	else if ((pwp = getpwnam(user)))
 	{
-	    user = pw->pw_name;
-	    home = pw->pw_dir;
+	    /*
+	     * This logic is needed to handle gracefully the possibility
+	     * that multiple names might be mapped to one UID
+	     */
+	    memcpy(&by_name, pwp, sizeof(struct passwd));
+
+	    if (by_name.pw_uid == by_uid.pw_uid)
+		pwp = &by_name;
+	    else
+		pwp = &by_uid;
 	}
 	else
 	{
@@ -57,8 +69,17 @@ void envquery(int argc, char **argv)
 		    program_name);
 	    exit(PS_UNDEFINED);
 	}
+	user = xstrdup(pwp->pw_name);
     }
 
+    if (!(home = getenv("HOME")))
+	home = pwp->pw_dir;
+
+    if ((program_name = strrchr(argv[0], '/')) != NULL)
+	++program_name;
+    else
+	program_name = argv[0];
+
 #define RCFILE_NAME	".fetchmailrc"
     rcfile = (char *) xmalloc(strlen(home)+strlen(RCFILE_NAME)+2);
     /* avoid //.fetchmailrc */
@@ -137,7 +158,7 @@ char *rfc822timestamp(void)
 /* return a timestamp in RFC822 form */
 {
     time_t	now;
-    static char buf[40];
+    static char buf[50];
 
     time(&now);
 #ifdef HAVE_STRFTIME
@@ -168,20 +189,22 @@ const char *showproto(int proto)
 {
     switch (proto)
     {
-    case P_AUTO: return("auto"); break;
+    case P_AUTO: return("auto");
 #ifdef POP2_ENABLE
-    case P_POP2: return("POP2"); break;
+    case P_POP2: return("POP2");
 #endif /* POP2_ENABLE */
-    case P_POP3: return("POP3"); break;
-    case P_IMAP: return("IMAP"); break;
-    case P_IMAP_K4: return("IMAP-K4"); break;
+    case P_POP3: return("POP3");
+    case P_IMAP: return("IMAP");
+    case P_IMAP_K4: return("IMAP-K4");
 #ifdef GSSAPI
-    case P_IMAP_GSS: return("IMAP-GSS"); break;
+    case P_IMAP_GSS: return("IMAP-GSS");
 #endif /* GSSAPI */
-    case P_APOP: return("APOP"); break;
-    case P_RPOP: return("RPOP"); break;
-    case P_ETRN: return("ETRN"); break;
-    default: return("unknown?!?"); break;
+    case P_IMAP_CRAM_MD5: return("IMAP-LOGIN");
+    case P_IMAP_LOGIN: return("IMAP-LOGIN");
+    case P_APOP: return("APOP");
+    case P_RPOP: return("RPOP");
+    case P_ETRN: return("ETRN");
+    default: return("unknown?!?");
     }
 }
 
