@@ -9,12 +9,14 @@
 #define		P_AUTO		1
 #define		P_POP2		2
 #define		P_POP3		3
-#define		P_IMAP		4
-#define		P_IMAP_K4	5
-#define		P_IMAP_GSS	6
-#define		P_APOP		7
-#define		P_RPOP		8
-#define		P_ETRN		9
+#define		P_APOP		4
+#define		P_RPOP		5
+#define		P_IMAP		6
+#define		P_IMAP_K4	7
+#define		P_IMAP_GSS	8
+#define		P_IMAP_CRAM_MD5	9
+#define		P_IMAP_LOGIN	10
+#define		P_ETRN		11
 
 #if INET6
 #define		SMTP_PORT	"smtp"
@@ -51,6 +53,7 @@
  */
 #define		MSGBUFSIZE	8192
 
+#define		NAMELEN		64	/* max username length */
 #define		PASSWORDLEN	64	/* max password length */
 #define		DIGESTLEN	33	/* length of MD5 digest */
 
@@ -68,6 +71,7 @@
 #define		PS_SMTP         10      /* SMTP error */
 #define		PS_DNS		11	/* fatal DNS error */
 #define		PS_BSMTP	12	/* output batch could not be opened */
+#define		PS_MAXFETCH	13	/* poll ended by fetch limit */
 /* leave space for more codes */
 #define		PS_UNDEFINED	23	/* something I hadn't thought of */
 #define		PS_TRANSIENT	24	/* transient failure (internal use) */
@@ -103,9 +107,11 @@ struct runctl
     char	*logfile;
     char	*idfile;
     int		poll_interval;
+    char	*postmaster;
+    flag	bouncemail;
+    char	*properties;
     flag	use_syslog;
     flag	invisible;
-    char	*postmaster;
 };
 
 struct idlist
@@ -142,7 +148,7 @@ struct method		/* describe methods for protocol state machine */
     flag delimited;		/* if true, accept "." message delimiter */
     int (*parse_response)(int, char *);
 				/* response_parsing function */
-    int (*password_canonify)(char *, char *);
+    int (*password_canonify)(char *, char *, int);
 				/* canonicalize password */
     int (*getauth)(int, struct query *, char *);
 				/* authorization fetcher */
@@ -194,7 +200,7 @@ struct hostdata		/* shared among all user connections to given server */
     flag checkalias;                  	/* resolve aliases by comparing IPs? */
 
 
-#ifdef linux
+#if defined(linux) || defined(__FreeBSD__)
     char *interface;
     char *monitor;
     int  monitor_io;
@@ -257,12 +263,14 @@ struct query
     flag active;		/* should we actually poll this server? */
     const char *destaddr;	/* destination host for this query */
     int errcount;		/* count transient errors in last pass */
+    int authfailcount;		/* count of authorization failures */
     int wedged;			/* wedged by auth failures or timeouts? */
     char *smtphost;		/* actual SMTP host we connected to */
     int smtp_socket;		/* socket descriptor for SMTP connection */
     unsigned int uid;		/* UID of user to deliver to */
     struct idlist *skipped;	/* messages skipped on the mail server */
     struct idlist *oldsaved, *newsaved;
+    char *lastid;		/* last Message-ID seen on this connection */
 
     /* internal use -- per-message state */
     int mimemsg;		/* bitmask indicating MIME body-type */
@@ -283,9 +291,12 @@ struct msgblk			/* message header parsed for open_sink() */
 
 /*
  * Numeric option handling.  Numeric option value of zero actually means
- * it's unspecified.  Value less than zero is zero.
+ * it's unspecified.  Value less than zero is zero.  The reason for this
+ * screwy encoding is so we can zero out an option block in order to set the
+ * numeric flags in it to unspecified.
  */
-#define NUM_VALUE(n)		(((n) == 0) ? -1 : (n))
+#define NUM_VALUE_IN(n)		(((n) == 0) ? -1 : (n))
+#define NUM_VALUE_OUT(n)	(((n) < 0) ? 0 : (n))
 #define NUM_NONZERO(n)		((n) > 0)
 #define NUM_ZERO(n)		((n) < 0)
 #define NUM_SPECIFIED(n)	((n) != 0)
@@ -332,15 +343,19 @@ extern int pass;		/* number of re-polling pass */
 extern flag configdump;		/* dump control blocks as Python dictionary */
 extern const char *fetchmailhost;
 				/* either "localhost" or an FQDN */
+#ifdef SDPS_ENABLE
+extern char *sdps_envfrom;
+extern char *sdps_envto;
+#endif /* SDPS_ENABLE */
 
 /* prototypes for globally callable functions */
 
 /* error.c: Error reporting */
 #if defined(HAVE_STDARG_H)
 void report_init(int foreground);
-void report (FILE *fp, int errnum, const char *format, ...);
+void report (FILE *fp, const char *format, ...);
 void report_build (FILE *fp, const char *format, ...);
-void report_complete (FILE *fp, int errnum, const char *format, ...);
+void report_complete (FILE *fp, const char *format, ...);
 void report_at_line (FILE *fp, int, const char *, unsigned int, const char *, ...);
 #else
 void report ();
@@ -426,9 +441,10 @@ int from64tobits(char *, const char *);
 #define MSG_NEEDS_DECODE  0x80
 extern void UnMimeHeader(unsigned char *buf);
 extern int  MimeBodyType(unsigned char *hdrs, int WantDecode);
-extern int  UnMimeBodyline(unsigned char **buf, int collapsedoubledot);
+extern int  UnMimeBodyline(unsigned char **buf, flag delimited, flag issoftline);
 
 /* interface.c */
+void interface_init(void);
 void interface_parse(char *, struct hostdata *);
 void interface_note_activity(struct hostdata *);
 int interface_approve(struct hostdata *);
@@ -449,7 +465,7 @@ char *xstrdup(const char *);
  #pragma alloca
 #endif
 #endif
-#define	xalloca(ptr, t, n)	if (!(ptr = (t) alloca(n))) report(stderr, PS_UNDEFINED, 0, "alloca failed")
+#define	xalloca(ptr, t, n)	if (!(ptr = (t) alloca(n))) {report(stderr, "alloca failed"); exit(PS_UNDEFINED);}
 
 /* protocol driver and methods */
 int do_protocol(struct query *, const struct method *);
@@ -472,9 +488,6 @@ void dump_config(struct runctl *runp, struct query *querylist);
 int is_host_alias(const char *, struct query *);
 char *host_fqdn(void);
 char *rfc822timestamp(void);
-#ifdef SDPS_ENABLE
-char *sdps_envto;
-#endif /* SDPS_ENABLE */
 
 void yyerror(const char *);
 int yylex(void);
