@@ -12,11 +12,13 @@ $wwwhost = $ENV{'WWWVIRTUAL'};
 my($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime(time);
 $month = $months[$mon];
 
+$century = 19 + int($year / 100);
+
 print <<EOF;
 Begin3
 Title:		fetchmail
 Version:	$version
-Entered-date:	${mday}${month}${year}
+Entered-date:	${mday}${month}${century}${year}
 Description:	Full-featured IMAP/POP2/POP3/APOP/RPOP/KPOP client with GUI
 		configuration, daemon mode, forwarding via SMTP or local MDA,
 		superior reply handling, support for multidrop mailboxes. 
@@ -24,7 +26,8 @@ Description:	Full-featured IMAP/POP2/POP3/APOP/RPOP/KPOP client with GUI
 		forwards fetched mail to your local delivery system. 
 		Your one-stop solution for intermittent email connections.
 		Home page and FAQ at http://$wwwhost/~esr/fetchmail.
-Keywords:	mail, client, POP, POP2, POP3, APOP, RPOP, KPOP, IMAP, SMTP
+Keywords:	mail, client, POP, POP2, POP3, APOP, RPOP, KPOP, IMAP, ETRN,
+    		SMTP, RPA, NTLM.
 Author: 	esr\@snark.thyrsus.com (Eric S. Raymond)
 Primary-site:	locke.ccil.org /pub/esr/fetchmail
 		${size}K fetchmail-$version.tar.gz
