@@ -37,38 +37,39 @@
 #define LA_RCFILE	13
 #define LA_IDFILE	14
 #define LA_POSTMASTER	15
-#define LA_PROTOCOL	16
-#define LA_UIDL		17
-#define LA_PORT		18
-#define LA_AUTHENTICATE	19
-#define LA_TIMEOUT	20
-#define LA_ENVELOPE	21
-#define LA_QVIRTUAL     22
-#define LA_USERNAME	23
-#define LA_ALL          24
-#define LA_NOKEEP	25
-#define	LA_KEEP		26
-#define LA_FLUSH        27
-#define LA_NOREWRITE	28
-#define LA_LIMIT	29
-#define LA_WARNINGS	30
-#define LA_FOLDER	31
-#define LA_SMTPHOST	32
-#define LA_SMTPADDR     33
-#define LA_ANTISPAM	34
-#define LA_BATCHLIMIT	35
-#define LA_FETCHLIMIT	36
-#define LA_EXPUNGE	37
-#define LA_MDA		38
-#define LA_BSMTP	39
-#define LA_LMTP		40
-#define LA_PLUGIN	41
-#define LA_PLUGOUT	42
-#define LA_NETSEC	43
-#define LA_INTERFACE    44
-#define LA_MONITOR      45
-#define LA_CONFIGDUMP	46
-#define LA_YYDEBUG	47
+#define LA_NOBOUNCE	16
+#define LA_PROTOCOL	17
+#define LA_UIDL		18
+#define LA_PORT		19
+#define LA_PREAUTH	20
+#define LA_TIMEOUT	21
+#define LA_ENVELOPE	22
+#define LA_QVIRTUAL     23
+#define LA_USERNAME	24
+#define LA_ALL          25
+#define LA_NOKEEP	26
+#define	LA_KEEP		27
+#define LA_FLUSH        28
+#define LA_NOREWRITE	29
+#define LA_LIMIT	30
+#define LA_WARNINGS	31
+#define LA_FOLDER	32
+#define LA_SMTPHOST	33
+#define LA_SMTPADDR     34
+#define LA_ANTISPAM	35
+#define LA_BATCHLIMIT	36
+#define LA_FETCHLIMIT	37
+#define LA_EXPUNGE	38
+#define LA_MDA		39
+#define LA_BSMTP	40
+#define LA_LMTP		41
+#define LA_PLUGIN	42
+#define LA_PLUGOUT	43
+#define LA_NETSEC	44
+#define LA_INTERFACE    45
+#define LA_MONITOR      46
+#define LA_CONFIGDUMP	47
+#define LA_YYDEBUG	48
 
 /* options still left: CDgGhHjJoORwWxXYz */
 static const char *shortoptions = 
@@ -91,12 +92,13 @@ static const struct option longoptions[] = {
   {"fetchmailrc",required_argument,(int *) 0, LA_RCFILE      },
   {"idfile",	required_argument, (int *) 0, LA_IDFILE      },
   {"postmaster",required_argument, (int *) 0, LA_POSTMASTER  },
+  {"nobounce",  no_argument,       (int *) 0, LA_NOBOUNCE    },
 
   {"protocol",	required_argument, (int *) 0, LA_PROTOCOL    },
   {"proto",	required_argument, (int *) 0, LA_PROTOCOL    },
   {"uidl",	no_argument,	   (int *) 0, LA_UIDL	     },
   {"port",	required_argument, (int *) 0, LA_PORT        },
-  {"auth",	required_argument, (int *) 0, LA_AUTHENTICATE},
+  {"preauth",	required_argument, (int *) 0, LA_PREAUTH},
   {"timeout",	required_argument, (int *) 0, LA_TIMEOUT     },
   {"envelope",	required_argument, (int *) 0, LA_ENVELOPE    },
   {"qvirtual",	required_argument, (int *) 0, LA_QVIRTUAL    },
@@ -128,10 +130,10 @@ static const struct option longoptions[] = {
   {"netsec",	required_argument, (int *) 0, LA_NETSEC      },
 #endif /* INET6 */
 
-#if defined(linux) && !INET6
+#if (defined(linux) && !INET6) || defined(__FreeBSD__)
   {"interface",	required_argument, (int *) 0, LA_INTERFACE   },
   {"monitor",	required_argument, (int *) 0, LA_MONITOR     },
-#endif /* defined(linux) && !INET6 */
+#endif /* (defined(linux) && !INET6) || defined(__FreeBSD__) */
   {"plugin",	required_argument, (int *) 0, LA_PLUGIN      },
   {"plugout",	required_argument, (int *) 0, LA_PLUGOUT     },
 
@@ -297,6 +299,9 @@ struct query *ctl;	/* option record to be initialized */
 	    rctl->postmaster = (char *) xmalloc(strlen(optarg)+1);
 	    strcpy(rctl->postmaster,optarg);
 	    break;
+	case LA_NOBOUNCE:
+	    run.bouncemail = FALSE;
+	    break;
 	case 'p':
 	case LA_PROTOCOL:
 	    /* XXX -- should probably use a table lookup here */
@@ -341,6 +346,10 @@ struct query *ctl;	/* option record to be initialized */
 	    else if (strcasecmp(optarg, "imap-gss") == 0)
                 ctl->server.protocol = P_IMAP_GSS;
 #endif /* GSSAPI */
+	    else if (strcasecmp(optarg, "imap-crammd5") == 0)
+                ctl->server.protocol = P_IMAP_CRAM_MD5;
+	    else if (strcasecmp(optarg, "imap-login") == 0)
+                ctl->server.protocol = P_IMAP_LOGIN;
 	    else if (strcasecmp(optarg,"etrn") == 0)
 		ctl->server.protocol = P_ETRN;
 	    else {
@@ -360,8 +369,7 @@ struct query *ctl;	/* option record to be initialized */
 	    ctl->server.port = xatoi(optarg, &errflag);
 #endif /* INET6 */
 	    break;
-	case 'A':
-	case LA_AUTHENTICATE:
+	case LA_PREAUTH:
 	    if (strcmp(optarg, "password") == 0)
 		ctl->server.preauthenticate = A_PASSWORD;
 	    else if (strcmp(optarg, "kerberos") == 0)
@@ -421,11 +429,11 @@ struct query *ctl;	/* option record to be initialized */
 	case 'l':
 	case LA_LIMIT:
 	    c = xatoi(optarg, &errflag);
-	    ctl->limit = NUM_VALUE(c);
+	    ctl->limit = NUM_VALUE_IN(c);
 	    break;
 	case 'r':
 	case LA_FOLDER:
-	    xalloca(buf, char *, strlen(optarg));
+	    xalloca(buf, char *, strlen(optarg) + 1);
 	    strcpy(buf, optarg);
 	    cp = strtok(buf, ",");
 	    do {
@@ -435,7 +443,7 @@ struct query *ctl;	/* option record to be initialized */
 	    break;
 	case 'S':
 	case LA_SMTPHOST:
-	    xalloca(buf, char *, strlen(optarg));
+	    xalloca(buf, char *, strlen(optarg) + 1);
 	    strcpy(buf, optarg);
 	    cp = strtok(buf, ",");
 	    do {
@@ -450,7 +458,7 @@ struct query *ctl;	/* option record to be initialized */
 	    break;
 	case 'Z':
 	case LA_ANTISPAM:
-	    xalloca(buf, char *, strlen(optarg));
+	    xalloca(buf, char *, strlen(optarg) + 1);
 	    strcpy(buf, optarg);
 	    cp = strtok(buf, ",");
 	    do {
@@ -463,17 +471,17 @@ struct query *ctl;	/* option record to be initialized */
 	case 'b':
 	case LA_BATCHLIMIT:
 	    c = xatoi(optarg, &errflag);
-	    ctl->batchlimit = NUM_VALUE(c);
+	    ctl->batchlimit = NUM_VALUE_IN(c);
 	    break;
 	case 'B':
 	case LA_FETCHLIMIT:
 	    c = xatoi(optarg, &errflag);
-	    ctl->fetchlimit = NUM_VALUE(c);
+	    ctl->fetchlimit = NUM_VALUE_IN(c);
 	    break;
 	case 'e':
 	case LA_EXPUNGE:
 	    c = xatoi(optarg, &errflag);
-	    ctl->expunge = NUM_VALUE(c);
+	    ctl->expunge = NUM_VALUE_IN(c);
 	    break;
 	case 'm':
 	case LA_MDA:
@@ -498,7 +506,7 @@ struct query *ctl;	/* option record to be initialized */
 #endif /* NET_SECURITY */
 	    break;
 
-#if defined(linux) && !INET6
+#if (defined(linux) && !INET6) || defined(__FreeBSD__)
 	case 'I':
 	case LA_INTERFACE:
 	    interface_parse(optarg, &ctl->server);
@@ -507,7 +515,7 @@ struct query *ctl;	/* option record to be initialized */
 	case LA_MONITOR:
 	    ctl->server.monitor = xstrdup(optarg);
 	    break;
-#endif /* defined(linux) && !INET6 */
+#endif /* (defined(linux) && !INET6) || defined(__FreeBSD__) */
 	case LA_PLUGIN:
 	    ctl->server.plugin = xstrdup(optarg);
 	    break;
@@ -523,7 +531,7 @@ struct query *ctl;	/* option record to be initialized */
 	case 'w':
 	case LA_WARNINGS:
 	    c = xatoi(optarg, &errflag);
-	    ctl->warnings = NUM_VALUE(c);
+	    ctl->warnings = NUM_VALUE_IN(c);
 	    break;
 
 	case LA_CONFIGDUMP:
@@ -565,7 +573,8 @@ struct query *ctl;	/* option record to be initialized */
 	P(_("  -f, --fetchmailrc specify alternate run control file\n"));
 	P(_("  -i, --idfile      specify alternate UIDs file\n"));
 	P(_("      --postmaster  specify recipient of last resort\n"));
-#if defined(linux) && !INET6
+	P(_("      --nobounce    redirect bounces from user to postmaster.\n"));
+#if (defined(linux) && !INET6) || defined(__FreeBSD__)
 	P(_("  -I, --interface   interface required specification\n"));
 	P(_("  -M, --monitor     monitor interface for activity\n"));
 #endif
@@ -575,7 +584,7 @@ struct query *ctl;	/* option record to be initialized */
 	P(_("  -p, --protocol    specify retrieval protocol (see man page)\n"));
 	P(_("  -U, --uidl        force the use of UIDLs (pop3 only)\n"));
 	P(_("  -P, --port        TCP/IP service port to connect to\n"));
-	P(_("  -A, --auth        authentication type (password or kerberos)\n"));
+	P(_("      --preauth     preauthentication type (password or kerberos)\n"));
 	P(_("  -t, --timeout     server nonresponse timeout\n"));
 	P(_("  -E, --envelope    envelope address header\n"));
 	P(_("  -Q, --qvirtual    prefix to remove from local user id\n"));
@@ -598,7 +607,7 @@ struct query *ctl;	/* option record to be initialized */
 	P(_("  -b, --batchlimit  set batch limit for SMTP connections\n"));
 	P(_("  -B, --fetchlimit  set fetch limit for server connections\n"));
 	P(_("  -e, --expunge     set max deletions between expunges\n"));
-        P(_("      --mda         set MDA to use for forwarding\n"));
+        P(_("  -m, --mda         set MDA to use for forwarding\n"));
         P(_("      --bsmtp       set output BSMTP file\n"));
         P(_("      --lmtp        use LMTP (RFC2033) for delivery\n"));
 	P(_("  -r, --folder      specify remote folder name\n"));
