@@ -37,7 +37,7 @@ static int pop3_phase;
 #define PHASE_LOGOUT	4
 static int last;
 #ifdef SDPS_ENABLE
-static flag sdps_enable = FALSE;
+char *sdps_envfrom;
 char *sdps_envto;
 #endif /* SDPS_ENABLE */
 
@@ -79,20 +79,32 @@ int pop3_ok (int sock, char *argbuf)
 		ok = PS_PROTOCOL;
 	    /*
 	     * We're checking for "lock busy", "unable to lock", 
-	     * "already locked" etc. here.  This indicates that we
-	     * have to wait for the server to clean up before we
-	     * can poll again.
+	     * "already locked", "wait a few minutes" etc. here. 
+	     * This indicates that we have to wait for the server to
+	     * unwedge itself before we can poll again.
 	     *
 	     * PS_LOCKBUSY check empirically verified with two recent
-	     * versions the Berkeley popper;	QPOP (version 2.2)  and
+	     * versions of the Berkeley popper; QPOP (version 2.2)  and
 	     * QUALCOMM Pop server derived from UCB (version 2.1.4-R3)
+	     * These are caught by the case-indifferent "lock" check.
+	     * The "wait" catches "mail storage services unavailable,
+	     * wait a few minutes and try again" on the InterMail server.
+	     *
+	     * If these aren't picked up on correctly, fetchmail will 
+	     * think there is an authentication failure and wedge the
+	     * connection in order to prevent futile polls.
+	     *
+	     * Gad, what a kluge.
 	     */
-	    else if (strstr(bufp,"lock")||strstr(bufp,"Lock")||strstr(bufp,"LOCK"))
+	    else if (strstr(bufp,"lock")
+		     || strstr(bufp,"Lock")
+		     || strstr(bufp,"LOCK")
+		     || strstr(bufp,"wait"))
 		ok = PS_LOCKBUSY;
 	    else
 		ok = PS_AUTHFAIL;
 	    if (*bufp)
-	      report(stderr, 0, bufp);
+	      report(stderr, "%s\n", bufp);
 	}
 	else
 	    ok = PS_PROTOCOL;
@@ -122,53 +134,39 @@ int pop3_getauth(int sock, struct query *ctl, char *greeting)
      * If we see either, and we're in multidrop mode, try to use
      * the SDPS *ENV extension.
      */
-    sdps_enable = (MULTIDROP(ctl) && strstr(greeting, "demon."));
-    /*
-     * Use SDPS if configured, regardless of the greeting string
-     * returned from the POP server. (Users accessing demon by a
-     * POP3 proxy may need this)
-     */
-    if (ctl->server.sdps)
-        sdps_enable = ctl->server.sdps;
+    if (!(ctl->server.sdps) && MULTIDROP(ctl) && strstr(greeting, "demon."))
+        ctl->server.sdps = TRUE;
 #endif /* SDPS_ENABLE */
 
     switch (ctl->server.protocol) {
     case P_POP3:
-	 ok = gen_transact(sock, "USER %s", ctl->remotename);
-
 #ifdef RPA_ENABLE
-	 /*
-	  * CompuServe has changed its RPA behavior.  Used to be they didn't
-	  * accept PASS, but I'm told this changed in mid-November 1997.
-	  */
-	 if (strstr(greeting, "csi.com")
-	     && (start = strchr(ctl->remotename, '@'))
-	     && !strcmp("@compuserve.com", start))
-	 {
-             /* temporary fix to get back out of cleartext authentication */
-             gen_transact(sock, "PASS %s", "dummypass");
-  
-	     /* AUTH command should return a list of available mechanisms */
-	     if (gen_transact(sock, "AUTH") == 0)
-	     {
-		 char buffer[10];
-		 flag has_rpa = FALSE;
+	/* CompuServe POP3 Servers as of 990730 want AUTH first for RPA */
+	if (strstr(ctl->remotename, "@compuserve.com"))
+	{
+	    /* AUTH command should return a list of available mechanisms */
+	    if (gen_transact(sock, "AUTH") == 0)
+	    {
+		char buffer[10];
+		flag has_rpa = FALSE;
 
-		 while ((ok = gen_recv(sock, buffer, sizeof(buffer))) == 0)
-		 {
-		     if (buffer[0] == '.')
-			 break;
-		     if (strncasecmp(buffer, "rpa", 3) == 0)
-			 has_rpa = TRUE;
-		 }
-		 if (has_rpa && !POP3_auth_rpa(ctl->remotename, 
-					       ctl->password, sock))
-		     return(PS_SUCCESS);
-	     }
+		while ((ok = gen_recv(sock, buffer, sizeof(buffer))) == 0)
+		{
+		    if (buffer[0] == '.')
+			break;
+		    if (strncasecmp(buffer, "rpa", 3) == 0)
+			has_rpa = TRUE;
+		}
+		if (has_rpa && !POP3_auth_rpa(ctl->remotename, 
+					      ctl->password, sock))
+		    return(PS_SUCCESS);
+	    }
 
-	     return(PS_AUTHFAIL);
-	 }
+	    return(PS_AUTHFAIL);
+	}
+	else /* not a CompuServe account */
 #endif /* RPA_ENABLE */
+	    ok = gen_transact(sock, "USER %s", ctl->remotename);
 
 #if OPIE
 	/* see RFC1938: A One-Time Password System */
@@ -205,7 +203,8 @@ int pop3_getauth(int sock, struct query *ctl, char *greeting)
 	for (start = greeting;  *start != 0 && *start != '<';  start++)
 	    continue;
 	if (*start == 0) {
-	    report(stderr, 0, _("Required APOP timestamp not found in greeting"));
+	    report(stderr,
+		   _("Required APOP timestamp not found in greeting\n"));
 	    return(PS_AUTHFAIL);
 	}
 
@@ -213,7 +212,8 @@ int pop3_getauth(int sock, struct query *ctl, char *greeting)
 	for (end = start;  *end != 0  && *end != '>';  end++)
 	    continue;
 	if (*end == 0 || end == start + 1) {
-	    report(stderr, 0, _("Timestamp syntax error in greeting"));
+	    report(stderr, 
+		   _("Timestamp syntax error in greeting\n"));
 	    return(PS_AUTHFAIL);
 	}
 	else
@@ -235,7 +235,7 @@ int pop3_getauth(int sock, struct query *ctl, char *greeting)
 	break;
 
     default:
-	report(stderr, 0, _("Undefined protocol request in POP3_auth"));
+	report(stderr, _("Undefined protocol request in POP3_auth\n"));
 	ok = PS_ERROR;
     }
 
@@ -243,7 +243,7 @@ int pop3_getauth(int sock, struct query *ctl, char *greeting)
     {
 	/* maybe we detected a lock-busy condition? */
         if (ok == PS_LOCKBUSY)
-	    report(stderr, 0, _("lock busy!  Is another session active?")); 
+	    report(stderr, _("lock busy!  Is another session active?\n")); 
 
 	return(ok);
     }
@@ -351,8 +351,8 @@ pop3_slowuidl( int sock,  struct query *ctl, int *countp, int *newp)
 		    try_id--;
 		}
 	    } else {
-		report(stderr, 0, 
-		       _("Messages inserted into list on server. Cannot handle this."));
+		report(stderr, 
+		       _("Messages inserted into list on server. Cannot handle this.\n"));
 		return -1;
 	    }
 	} 
@@ -425,7 +425,7 @@ static int pop3_getrange(int sock,
 	{
 	    if (sscanf(buf, "%d", &last) == 0)
 	    {
-		report(stderr, 0, _("protocol error"));
+		report(stderr, _("protocol error\n"));
 		return(PS_ERROR);
 	    }
 	    *newp = (*countp - last);
@@ -438,7 +438,7 @@ static int pop3_getrange(int sock,
 		/* don't worry, yet! do it the slow way */
 		if((ok = pop3_slowuidl( sock, ctl, countp, newp))!=0)
 		{
-		    report(stderr, 0, _("protocol error while fetching UIDLs"));
+		    report(stderr, _("protocol error while fetching UIDLs\n"));
 		    return(PS_ERROR);
 		}
 	    }
@@ -548,10 +548,11 @@ static int pop3_fetch(int sock, struct query *ctl, int number, int *lenp)
      * See http://www.demon.net/services/mail/sdps-tech.html
      * for a description of what we're parsing here.
      */
-    if (sdps_enable)
+    if (ctl->server.sdps)
     {
 	int	linecount = 0;
 
+	sdps_envfrom = (char *)NULL;
 	sdps_envto = (char *)NULL;
 	gen_send(sock, "*ENV %d", number);
 	do {
@@ -560,26 +561,33 @@ static int pop3_fetch(int sock, struct query *ctl, int number, int *lenp)
                 break;
             }
             linecount++;
-            if (linecount == 5)
-            {
+	    switch (linecount) {
+	    case 4:
+		/* No need to wrap envelope from address */
+		sdps_envfrom = xmalloc(strlen(buf)+1);
+		strcpy(sdps_envfrom,buf);
+		break;
+	    case 5:
                 /* Wrap address with To: <> so nxtaddr() likes it */
                 sdps_envto = xmalloc(strlen(buf)+7);
                 sprintf(sdps_envto,"To: <%s>",buf);
+		break;
             }
 	} while
-	    (buf[0] !='.');
+	    (!(buf[0] == '.' && (buf[1] == '\r' || buf[1] == '\n' || buf[1] == '\0')));
     }
 #endif /* SDPS_ENABLE */
 
     /*
-     * Though the POP RFCs don't document this fact, on every POP3 server
-     * I know of messages are marked "seen" only at the time the OK
-     * response to a RETR is issued.
+     * Though the POP RFCs don't document this fact, on almost every
+     * POP3 server I know of messages are marked "seen" only at the
+     * time the OK response to a RETR is issued.
      *
      * This means we can use TOP to fetch the message without setting its
      * seen flag.  This is good!  It means that if the protocol exchange
      * craps out during the message, it will still be marked `unseen' on
-     * the server.
+     * the server.  (Exception: in early 1999 SpryNet's POP3 servers were
+     * reported to mark messages seen on a TOP fetch.)
      *
      * However...*don't* do this if we're using keep to suppress deletion!
      * In that case, marking the seen flag is the only way to prevent the
@@ -598,8 +606,7 @@ static int pop3_fetch(int sock, struct query *ctl, int number, int *lenp)
      *
      * The line count passed (99999999) is the maximum value CompuServe will
      * accept; it's much lower than the natural value 2147483646 (the maximum
-     * twos-complement signed 32-bit integer minus 1)
-     */
+     * twos-complement signed 32-bit integer minus 1) */
     if (ctl->keep || ctl->fetchall)
 	gen_send(sock, "RETR %d", number);
     else
