@@ -32,6 +32,9 @@
 #include "fetchmail.h"
 #include "i18n.h"
 
+/* We need to define h_errno only if it is not already */
+#ifndef h_errno
+
 #ifdef HAVE_RES_SEARCH
 /* some versions of FreeBSD should declare this but don't */
 extern int h_errno;
@@ -40,6 +43,8 @@ extern int h_errno;
 static int h_errno;
 #endif
 
+#endif /* ndef h_errno */
+
 #if NET_SECURITY
 #include <net/security.h>
 #endif /* NET_SECURITY */
@@ -52,13 +57,13 @@ static int handle_plugin(const char *host,
     int fds[2];
     if (socketpair(AF_UNIX,SOCK_STREAM,0,fds))
     {
-	report(stderr, errno, _("fetchmail: socketpair failed"));
+	report(stderr, _("fetchmail: socketpair failed\n"));
 	return -1;
     }
     switch (fork()) {
 	case -1:
 		/* error */
-		report(stderr, errno, _("fetchmail: fork failed"));
+		report(stderr, _("fetchmail: fork failed\n"));
 		return -1;
 		break;
 	case 0:	/* child */
@@ -66,15 +71,15 @@ static int handle_plugin(const char *host,
 		** detection */
 		(void) close(fds[1]);
 		if ( (dup2(fds[0],0) == -1) || (dup2(fds[0],1) == -1) ) {
-			report(stderr, errno, _("dup2 failed"));
+			report(stderr, _("dup2 failed\n"));
 			exit(1);
 		}
 		/* fds[0] is now connected to 0 and 1; close it */
 		(void) close(fds[0]);
 		if (outlevel >= O_VERBOSE)
-		    report(stderr, 0, _("running %s %s %s"), plugin, host, service);
+		    report(stderr, _("running %s %s %s\n"), plugin, host, service);
 		execlp(plugin,plugin,host,service,0);
-		report(stderr, errno, _("execl(%s) failed"), plugin);
+		report(stderr, _("execl(%s) failed\n"), plugin);
 		exit(0);
 		break;
 	default:	/* parent */
@@ -87,11 +92,35 @@ static int handle_plugin(const char *host,
 }
 #endif /* HAVE_SOCKETPAIR */
 
+#ifdef __UNUSED__
+#include <sys/time.h>
+
+int SockCheckOpen(int fd)
+/* poll given socket; is it selectable? */
+{
+    fd_set r, w, e;
+    int rt;
+    struct timeval tv;
+  
+    for (;;) 
+    {
+	FD_ZERO(&r); FD_ZERO(&w); FD_ZERO(&e);
+	FD_SET(fd, &e);
+    
+	tv.tv_sec = 0; tv.tv_usec = 0;
+	rt = select(fd+1, &r, &w, &e, &tv);
+	if (rt == -1 && (errno != EAGAIN && errno != EINTR))
+	    return 0;
+	if (rt != -1)
+	    return 1;
+    }
+}
+#endif /* __UNUSED__ */
+
 #if INET6
 int SockOpen(const char *host, const char *service, const char *options,
 	     const char *plugin)
 {
-    int i;
     struct addrinfo *ai, req;
 #if NET_SECURITY
     void *request = NULL;
@@ -105,8 +134,8 @@ int SockOpen(const char *host, const char *service, const char *options,
     memset(&req, 0, sizeof(struct addrinfo));
     req.ai_socktype = SOCK_STREAM;
 
-    if (i = getaddrinfo(host, service, &req, &ai)) {
-	report(stderr, i, _("fetchmail: getaddrinfo(%s.%s)"), host,service);
+    if (getaddrinfo(host, service, &req, &ai)) {
+	report(stderr, _("fetchmail: getaddrinfo(%s.%s)\n"), host,service);
 	return -1;
     };
 
@@ -144,11 +173,11 @@ int SockOpen(const char *host, const char *service, const char *options,
 int SockOpen(const char *host, int clientPort, const char *options,
 	     const char *plugin)
 {
-    int sock;
+    int sock = -1;	/* pacify -Wall */
 #ifndef HAVE_INET_ATON
     unsigned long inaddr;
 #endif /* HAVE_INET_ATON */
-    struct sockaddr_in ad;
+    struct sockaddr_in ad, **pptr;
     struct hostent *hp;
 
 #ifdef HAVE_SOCKETPAIR
@@ -186,34 +215,40 @@ int SockOpen(const char *host, int clientPort, const char *options,
 	if(hp->h_length != 4 && hp->h_length != 8)
 	{
 	    h_errno = errno = 0;
-	    report(stderr, 0, _("fetchmail: illegal address length received for host %s"),host);
+	    report(stderr, 
+		   _("fetchmail: illegal address length received for host %s\n"),host);
 	    return -1;
 	}
 	/*
-	 * FIXME: make this work for multihomed hosts.
-	 * We're toast if we get back multiple addresses and h_addrs[0]
-	 * (aka h_addr) is not one we can actually connect to; this happens
-	 * with multi-homed boxen.
+	 * Try all addresses of a possibly multihomed host until we get
+	 * a successful connect or until we run out of addresses.
 	 */
-        memcpy(&ad.sin_addr, hp->h_addr, hp->h_length);
+	pptr = (struct sockaddr_in **)hp->h_addr_list;
+	for(; *pptr != NULL; pptr++)
+	{
+	    sock = socket(AF_INET, SOCK_STREAM, 0);
+	    if (sock < 0)
+	    {
+		h_errno = 0;
+		return -1;
+	    }
+	    ad.sin_port = htons(clientPort);
+	    memcpy(&ad.sin_addr, *pptr, sizeof(struct in_addr));
+	    if (connect(sock, (struct sockaddr *) &ad, sizeof(ad)) == 0)
+		break; /* success */
+	    close(sock);
+	    memset(&ad, 0, sizeof(ad));
+	    ad.sin_family = AF_INET;
+	}
+	if(*pptr == NULL)
+	{
+	    int olderr = errno;
+	    close(sock);
+	    h_errno = 0;
+	    errno = olderr;
+	    return -1;
+	}
     }
-    ad.sin_port = htons(clientPort);
-    
-    sock = socket(AF_INET, SOCK_STREAM, 0);
-    if (sock < 0)
-    {
-	h_errno = 0;
-        return -1;
-    }
-    if (connect(sock, (struct sockaddr *) &ad, sizeof(ad)) < 0)
-    {
-	int olderr = errno;
-	close(sock);
-	h_errno = 0;
-	errno = olderr;
-        return -1;
-    }
-
     return(sock);
 }
 #endif /* INET6 */
