@@ -13,16 +13,17 @@ Source:         %{name}-%{version}.tar.gz
 Group:		Applications/Mail
 Group(pt_BR):   Aplicaes/Correio Eletrnico
 Copyright:	GPL
-Icon:		fetchmail.gif
+Icon:		fetchmail.xpm
 Requires:	smtpdaemon
 BuildRoot:	/var/tmp/%{name}-%{version}
 Summary:	Full-featured POP/IMAP mail retrieval daemon
 Summary(fr):    Collecteur (POP/IMAP) de courrier lectronique
 Summary(de):    Program zum Abholen von E-Mail via POP/IMAP
-Summary(pt_BR): Busca mensagens de um servidor usando POP ou IMAP
-Summary(es_AR): Recolector de correo via POP/IMAP
+Summary(pt):    Busca mensagens de um servidor usando POP ou IMAP
+Summary(es):    Recolector de correo via POP/IMAP
 Summary(pl):    Zdalny demon pocztowy do protokow POP2, POP3, APOP, IMAP
 Summary(tr):    POP2, POP3, APOP, IMAP protokolleri ile uzaktan mektup alma yazlm
+Summary(da):    Alsidig POP/IMAP post-afhentnings dmon
 
 %description
 Fetchmail is a free, full-featured, robust, and well-documented remote
@@ -43,15 +44,15 @@ serveur SMTP local (habituellement sendmail).
 %description -l de
 Fetchmail ist ein freies, vollstndiges, robustes und
 wohldokumentiertes Werkzeug zum Abholen und Weiterreichen von E-Mail,
-gedacht zum Gebrauchber temporre TCP/IP-Verbindungen (wie
+gedacht zum Gebrauch ber temporre TCP/IP-Verbindungen (wie
 z.B. SLIP- oder PPP-Verbindungen).  Es holt E-Mail von (weit)
-entfernten Mail-Servern abund reicht sie an das Auslieferungssystem
+entfernten Mail-Servern ab und reicht sie an das Auslieferungssystem
 der lokalen Client-Maschine weiter, damit sie dann von normalen MUAs
 ("mail user agents") wie mutt, elm, pine, (x)emacs/gnus oder mailx
-gelesen werden kann.  Ein interaktiver GUI-Konfigurator auch gut
+gelesen werden knnen.  Ein interaktiver GUI-Konfigurator auch gut
 geeignet zum Gebrauch durch Endbenutzer wird mitgeliefert.
 
-%description -l pt_BR
+%description -l pt
 Fetchmail  um programa que  usado para recuperar mensagens de um
 servidor de mail remoto. Ele pode usar Post Office Protocol (POP)
 ou IMAP (Internet Mail Access Protocol) para isso, e entrega o mail
@@ -59,9 +60,9 @@ atravs do servidor local SMTP (normalmente sendmail).
 
 %description -l es
 Fetchmail es una utilidad gratis, completa, robusta y bien documentada
-para la recepcin y reeenvo de correo pensada para ser usada en co-
-nexiones TCP/IP por demanda (como SLIP y PPP). Recibe el correo de
-servidores remotos y lo reenva a el sistema de entrega local, siendo de
+para la recepcin y reenvo de correo pensada para ser usada en
+conexiones TCP/IP temporales (como SLIP y PPP). Recibe el correo de
+servidores remotos y lo reenva al sistema de entrega local, siendo de
 ese modo posible leerlo con programas como mutt, elm, pine, (x)emacs/gnus
 o mailx. Contiene un configurador GUI interactivo pensado para usuarios.
 
@@ -75,9 +76,21 @@ description -l tr
 fetchmail yazlm, POP veya IMAP destei veren bir sunucuda yer alan
 mektuplarnz alr.
 
+description -l da
+Fetchmail er et gratis, robust, alsidigt og vel-dokumenteret vrktj 
+til afhentning og videresending af elektronisk post via TCP/IP
+baserede opkalds-forbindelser (ssom SLIP eller PPP forbindelser).   
+Den henter post fra en ekstern post-server, og videresender den
+til din lokale klient-maskines post-system, s den kan lses af
+almindelige mail klienter ssom mutt, elm, pine, (x)emacs/gnus,
+eller mailx. Der medflger ogs et interaktivt GUI-baseret
+konfigurations-program, som kan bruges af almindelige brugere.
+
 %package -n fetchmailconf
 Summary:        A GUI configurator for generating fetchmail configuration files
 Summary(pl):    GUI konfigurator do fetchmaila
+Summary(fr):	GUI configurateur pour fetchmail
+Summary(es):	Configurador GUI interactivo para fetchmail
 Group:          Utilities/System
 Requires:       %{name} = %{version}, python
 
@@ -85,6 +98,12 @@ Requires:       %{name} = %{version}, python
 A GUI configurator for generating fetchmail configuration file written in
 python
 
+%description -n fetchmailconf -l es
+Configurador grfico para fetchmail escrito en python
+
+%description -n fetchmailconf -l de
+Ein interaktiver GUI-Konfigurator fr fetchmail in python
+
 %description -n fetchmailconf -l pl
 GUI konfigurator do fetchmaila napisany w pythonie.
 
@@ -92,9 +111,9 @@ GUI konfigurator do fetchmaila napisany w pythonie.
 %setup -q
 
 %build
-CFLAGS="$RPM_OPT_FLAGS" LDFLAGS="-s" \
-# Add  --enable-nls --without-included-gettext for internationalization
-./configure
+CFLAGS="$RPM_OPT_FLAGS" LDFLAGS="-s" # Add  --enable-nls --without-included-gettext for internationalization
+export CFLAGS LDFLAGS
+./configure -prefix=/usr
 make
 
 %install
