@@ -33,7 +33,7 @@
  * considered seen in `oldsaved'.  These are messages that were left in
  * the mailbox and *not deleted* on previous queries (we don't need to
  * remember the UIDs of deleted messages because ... well, they're gone!)
- * This list is initially set up by initialized_saved_list() from the
+ * This list is initially set up by initialize_saved_list() from the
  * .fetchids file.
  *
  * Early in the query, during the execution of the protocol-specific 
@@ -70,7 +70,6 @@ static struct idlist *scratchlist;
 void initialize_saved_lists(struct query *hostlist, const char *idfile)
 /* read file of saved IDs and attach to each host */
 {
-    int	st;
     FILE	*tmpfp;
     struct query *ctl;
 
@@ -79,32 +78,101 @@ void initialize_saved_lists(struct query *hostlist, const char *idfile)
 	ctl->skipped = ctl->oldsaved = ctl->newsaved = (struct idlist *)NULL;
 
     /* let's get stored message UIDs from previous queries */
-    if ((tmpfp = fopen(idfile, "r")) != (FILE *)NULL) {
-	char buf[POPBUFSIZE+1],host[HOSTLEN+1],user[USERNAMELEN+1],id[IDLEN+1];
+    if ((tmpfp = fopen(idfile, "r")) != (FILE *)NULL)
+    {
+	char buf[POPBUFSIZE+1];
+	char *host = NULL;	/* pacify -Wall */
+	char *user;
+	char *id;
+	char *atsign;	/* temp pointer used in parsing user and host */
+	char *delimp1;
+	char saveddelim1;
+	char *delimp2;
+	char saveddelim2 = '\0';	/* pacify -Wall */
 
 	while (fgets(buf, POPBUFSIZE, tmpfp) != (char *)NULL)
 	{
-	    /* possible lossage here with very old versions of sscanf(3)... */
-	    if ((st = sscanf(buf, "%[^@]@%s %s\n", user, host, id)) == 3)
-	    {
-		for (ctl = hostlist; ctl; ctl = ctl->next)
-		{
-		    if (ctl->server.truename &&
-		        strcasecmp(host, ctl->server.truename) == 0
-				&& strcasecmp(user, ctl->remotename) == 0)
-		    {
-			save_str(&ctl->oldsaved, id, UID_SEEN);
-			break;
-		    }
-		}
+	  /*
+	   * At this point, we assume the bug has two fields -- a user@host 
+	   * part, and an ID part. Either field may contain spurious @ signs. 
+           * The previous version of this code presumed one could split at 
+           * the rightmost '@'.  This is not correct, as InterMail puts an 
+           * '@' in the UIDL.
+	   */
+	  
+	  /* very first, skip leading spaces */
+	  user = buf + strspn(buf, " \t");
 
-		/* if it's not in a host we're querying, save it anyway */
-		if (ctl == (struct query *)NULL)
-		    save_str(&scratchlist, buf, UID_SEEN);
+	  /* First, we split the buf into a userhost part and an id part */
+	  if ( (delimp1 = strpbrk(user, " \t")) != NULL ) {
+		id = delimp1 + strspn(delimp1, " \t");	/* set pointer to id */
+	  	saveddelim1 = *delimp1;	/* save char after token */
+		*delimp1 = '\0';		/* delimit token with \0 */
+		if (id != NULL) {
+			/* now remove trainling white space chars from id */
+	  		if ( (delimp2 = strpbrk(id, " \t\n")) != NULL ) {
+				saveddelim2 = *delimp2;
+				*delimp2 = '\0';
+			}
+	      		atsign = strrchr(user, '@');
+	      		if (atsign) {
+				*atsign = '\0';
+				host = atsign + 1;
+
+			}
+			for (ctl = hostlist; ctl; ctl = ctl->next) {
+				if (ctl->server.truename &&
+			 	    strcasecmp(host, ctl->server.truename) == 0
+			 	    && strcasecmp(user, ctl->remotename) == 0) {
+	
+					save_str(&ctl->oldsaved, id, UID_SEEN);
+					break;
+				}
+			}
+			/* if it's not in a host we're querying,
+			** save it anyway */
+			if (ctl == (struct query *)NULL) {
+				/* restore string */
+				*delimp1 = saveddelim1;
+				*atsign = '@';
+	  			if (delimp2 != NULL) {
+					*delimp2 = saveddelim2;
+				}
+				save_str(&scratchlist, buf, UID_SEEN);
+			}
+		}
 	    }
 	}
 	fclose(tmpfp);
     }
+
+    if (outlevel >= O_DEBUG)
+    {
+	struct idlist	*idp;
+	int uidlcount = 0;
+
+	for (ctl = hostlist; ctl; ctl = ctl->next)
+	    if (ctl->server.uidl)
+	    {
+		report_build(stdout, "Old UID list from %s:",ctl->server.pollname);
+		for (idp = ctl->oldsaved; idp; idp = idp->next)
+		    report_build(stdout, " %s", idp->id);
+		if (!idp)
+		    report_build(stdout, " <empty>");
+		report_complete(stdout, "\n");
+		uidlcount++;
+	    }
+
+	if (uidlcount)
+	{
+	    report_build(stdout, "Scratch list of UIDs:");
+	    for (idp = scratchlist; idp; idp = idp->next)
+		report_build(stdout, " %s", idp->id);
+	    if (!idp)
+		report_build(stdout, " <empty>");
+	    report_complete(stdout, "\n");
+	}
+    }
 }
 #endif /* POP3_ENABLE */
 
@@ -304,8 +372,21 @@ void update_str_lists(struct query *ctl)
 /* perform end-of-query actions on UID lists */
 {
     free_str_list(&ctl->oldsaved);
+    free_str_list(&scratchlist);
     ctl->oldsaved = ctl->newsaved;
     ctl->newsaved = (struct idlist *) NULL;
+
+    if (ctl->server.uidl && outlevel >= O_DEBUG)
+    {
+	struct idlist *idp;
+
+	report_build(stdout, "New UID list from %s:", ctl->server.pollname);
+	for (idp = ctl->oldsaved; idp; idp = idp->next)
+	    report_build(stdout, " %s = %d", idp->id, idp->val.status.mark);
+	if (!idp)
+	    report_build(stdout, " <empty>");
+	report_complete(stdout, "\n");
+    }
 }
 
 void write_saved_lists(struct query *hostlist, const char *idfile)
@@ -324,8 +405,12 @@ void write_saved_lists(struct query *hostlist, const char *idfile)
     }
 
     /* either nuke the file or write updated last-seen IDs */
-    if (!idcount)
+    if (!idcount && !scratchlist)
+    {
+	if (outlevel >= O_DEBUG)
+	    report(stdout, "Deleting fetchids file.\n");
 	unlink(idfile);
+    }
     else
 	if ((tmpfp = fopen(idfile, "w")) != (FILE *)NULL) {
 	    for (ctl = hostlist; ctl; ctl = ctl->next) {
